%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2019-06-18
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/ustereo.html
%
% (c) 2019 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%

function [engine] = generateNumericData(geometryFullpath, lights, noiseStdDev, quantification)
noise = {};
noise.mean = 0;
noise.standard_deviation = noiseStdDev;
noise.variance = noiseStdDev^2;

engine = {};
engine.parameters = {};
engine.parameters.noise = noise;
engine.parameters.lights = lights;

fprintf('Reading geometry... ');
[~, filename, ext] = fileparts(geometryFullpath);
fprintf('Loading %s... ', [filename, ext])
    
fid = fopen(geometryFullpath);

line = fgets(fid);
parts = split(line, ' ');
min_theta = str2double(parts{1});
max_theta = str2double(parts{2});
min_phi = str2double(parts{3});
max_phi = str2double(parts{4});
lambda = str2double(parts{5});

incr = pi/lambda;

min_phi = ceil(min_phi/incr)*incr;
max_phi = floor(max_phi/incr)*incr;
min_theta = ceil(min_theta/incr)*incr;
max_theta = floor(max_theta/incr)*incr;

if max_theta >= pi/2
    max_theta = pi/2 - incr;
end
if max_phi >= 2*pi
    max_phi = 2*pi - incr;
end

theta = min_theta : incr : max_theta;
phi = min_phi : incr : max_phi;

geometry = cell(length(theta), length(phi));
X = zeros(size(geometry));
Y = zeros(size(geometry));
Z = zeros(size(geometry));
rho = zeros(size(geometry));

line = fgets(fid);
while ischar(line)
    if line(1) ~= '#'
        parts = split(line, ' ');
        
        i = str2double(parts{1}) + 1;
        j = str2double(parts{2}) + 1;

        geometry{i, j}.X = [ str2double(parts{3}), str2double(parts{4}), str2double(parts{5}) ];
        geometry{i, j}.N = [ str2double(parts{6}), str2double(parts{7}), str2double(parts{8}) ];
        rho(i, j) = norm(geometry{i, j}.X);
        
        if geometry{i, j}.N(3) > 0
            geometry{i, j}.N = -geometry{i, j}.N;
        end
        
        X(i, j) = str2double(parts{3});
        Y(i, j) = str2double(parts{4});
        Z(i, j) = str2double(parts{5});
    end
        
    line = fgets(fid);
end

engine.parameters.lambda = lambda;
engine.parameters.theta = theta;
engine.parameters.phi = phi;

engine.data = cell(size(geometry));

[~,lights_count] = size(lights);

rhoNorm = (rho - min(rho(:))) / (max(rho(:)) - min(rho(:)));

for i = 1:length(theta)
    for j = 1:length(phi)
        t = theta(i);
        p = phi(j);
        
        engine.data{i, j}.N = geometry{i, j}.N;
        engine.data{i, j}.X = geometry{i, j}.X;
        engine.data{i, j}.rho = rho(i, j);
        engine.data{i, j}.rhoNorm = rhoNorm(i, j);
        
        engine.data{i, j}.rawI = zeros(lights_count, 1);
        engine.data{i, j}.I = zeros(lights_count, 1);
                
        for l = 1:lights_count
            pd = lights(1, l);
            qd = lights(2, l);
            
            L = [pd, qd, -1];
            L = L/norm(L);
            
            if quantification
                engine.data{i, j}.rawI(l) = double(floor(dot(engine.data{i, j}.N, L) * 255)) / 255;
            else
                engine.data{i, j}.rawI(l) = dot(engine.data{i, j}.N, L);
            end
            
            engine.data{i, j}.I(l) = engine.data{i, j}.rawI(l) + rand() * noise.variance;
        end
    end
end

for l = 1:lights_count
    I = zeros(size(geometry));
    
    for i = 1:length(theta)
        for j = 1:length(phi)
            I(i, j) = engine.data{i, j}.I(l);
        end
    end
end

fprintf('OK!\n');
